package org.bitbucket.bradleysmithllc.etlunit.cli;

import org.clamshellcli.api.Command;
import org.clamshellcli.api.Configurator;
import org.clamshellcli.api.Context;
import org.clamshellcli.api.IOConsole;
import org.codehaus.jackson.JsonNode;
import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.node.ArrayNode;
import org.bitbucket.bradleysmithllc.etlunit.feature.Feature;
import org.bitbucket.bradleysmithllc.etlunit.maven.ETLUnitMojo;
import org.bitbucket.bradleysmithllc.etlunit.util.IOUtils;

import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class AddAvailableFeatureCmd implements Command {
	private static final String NAMESPACE = "syscmd";
	public static final String ACTION_NAME = "add-feature";

	public Object execute(Context ctx) {
		IOConsole console = ctx.getIoConsole();
		String [] inputLine = (String []) ctx.getValue(Context.KEY_COMMAND_LINE_ARGS);

		File cfg = new File(".");

		if (inputLine == null || inputLine.length != 1)
		{
			console.writeOutput("Please specify exactly one feature to add" + Configurator.VALUE_LINE_SEP);
			return null;
		}

		try {
			List<String> installedFeatures = ListInstalledFeaturesCmd.getInstalledFeatures(cfg);

			String featureName = inputLine[0];
			if (installedFeatures != null && installedFeatures.contains(featureName))
			{
				console.writeOutput("Feature " + featureName + " already added" + Configurator.VALUE_LINE_SEP);
			}
			else
			{
				List<Feature> availableFeatures = ListAvailableFeaturesCmd.getAvailableFeatures();

				Feature toInstall = null;

				for (Feature feature : availableFeatures)
				{
					if (feature.getFeatureName().equals(featureName))
					{
						toInstall = feature;
					}
				}

				if (toInstall != null)
				{
					// load the config file and pretty print it
					ObjectMapper mapper = new ObjectMapper();

					File etlunitConfiguration = ETLUnitMojo.getEtlunitConfiguration(cfg);
					JsonNode node = mapper.readTree("{" + IOUtils.readFileToString(etlunitConfiguration) + "}");

					( (ArrayNode) node.get("install-features")).add(featureName);

					String config = mapper.writerWithDefaultPrettyPrinter()
						.writeValueAsString(node);

					int index = config.indexOf("{");

					config = config.substring(index + 1);

					index = config.lastIndexOf("}");

					config = config.substring(0, index - 1).trim();

					IOUtils.writeBufferToFile(etlunitConfiguration, new StringBuffer(config));

					console.writeOutput("Feature " + featureName + " added" + Configurator.VALUE_LINE_SEP);
				}
				else
				{
					console.writeOutput("Feature " + featureName + " not available.  Run list-available-features for valid features to install" + Configurator.VALUE_LINE_SEP);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		return null;
	}

	public void plug(Context plug) {
		// no load-time setup needed
	}

	public Descriptor getDescriptor() {
		return new Descriptor() {
			public String getNamespace() {
				return NAMESPACE;
			}

			public String getName() {
				return ACTION_NAME;
			}

			public String getDescription() {
				return "Adds a known feature to the project (listed in " + ListAvailableFeaturesCmd.ACTION_NAME + ")";
			}

			public String getUsage() {
				return "";
			}

			public Map<String, String> getArguments() {
				Map<String, String> map = new HashMap<String, String>();

				return map;
			}
		};
	}
}